#############################################################################
#
#             Copyright (C) FarSite Communications Ltd. 2020
#                             All Rights Reserved
#
# License is granted to copy and use this software and to make and
# use derivative works.
#
# FarSite Communications Limited makes no representations concerning 
# either the merchantability of this software or the suitability of 
# this software for any particular purpose. It is provided "as is"
# without express or implied warranty of any kind.
#
# Program     : fsbertex1
#                                                                 
# File        : fsbert.py
#                                                                 
# Version     : V1.0.0
#                                                                   
# Description : Sample python class to present the COM-based FarSync BERT API
#
# Modifications
#
# Date      Init   Details
#
# 12Oct20   WEB    Initial version
#
#############################################################################

# Note: The FarSync BERT's configuration parameters are utilised when it is
#       started. Therefore the properties should be set BEFORE calling start()
#       However, the DTR and RTS states can be changed after the BERT has
#       been started

from comtypes.client import CreateObject, Constants
import time

# This is the FsBert class which presents the COM-based FarSync BERT API for
# used by python client apps

class FsBert:

    # FsBert Constuctor
    
    # device            - specifies the FarSync device instance to be used - this number is the same as
    #                     the # identified in Device Manager in FarSync WAN Adapters - FarSync Flex/T2Ee (SDCI#)
    # port              - specifies the port number of the selected FarSync device
    # internal_clocking - specifies when the FarSync port will use internal (or extenal) clocking (true/false)
    # rate              - specifies the rate of the line connected to the FarSync port. 

    def __init__(self, device, port, internal_clocking, rate):

        # create the FarSync BERT API COM object
        self.bert = CreateObject("FsBert.FsBert2")

        # configure the object as specified by caller
        self.bert.device = device
        self.bert.port = port
        self.bert.interface = Constants(self.bert).FsBertInterface_V24
        if internal_clocking:
            self.bert.clocking = Constants(self.bert).FsBertClocking_Internal
        else:
            self.bert.clocking = Constants(self.bert).FsBertClocking_External
        self.bert.rate = rate

        # now set any properties that have not specified by the (example) caller
        self.bert.pattern = Constants(self.bert).FsBertPattern_2_15
        self.bert.test_length = 0
        self.bert.logging_mode = True
        self.bert.software_mode = True
        # make the BertClient console visible
        self.bert.debugMode = 2
        self.bert.RTS = False
        self.bert.DTR = False

    # properties and setters for those features of the FarSyunc BERT API
    # that are required by the calling app

    @property
    def device(self) -> int:
        return self.bert.Device

    @device.setter
    def device(self, value: int):
        self.bert.Device = value

    @property
    def port(self) -> int:
        return self.bert.Port

    @port.setter
    def port(self, value: int):
        self.bert.Port = value

    @property
    def interface(self) -> int:
        return self.bert.Interface

    @interface.setter
    def interface(self, value: int):
        self.bert.Interface = value

    @property
    def clocking(self) -> int:
        return self.bert.Clocking

    @clocking.setter
    def clocking(self, value: int):
        self.bert.Clocking = value
        self.restart()

    @property
    def rate(self) -> int:
        return self.bert.Rate

    @rate.setter
    def rate(self, value: int):
        self.bert.Rate = value

    @property
    def pattern(self) -> int:
        return self.bert.Pattern

    @pattern.setter
    def pattern(self, value: int):
        self.bert.Pattern = value

    @property
    def test_length(self) -> int:
        return self.bert.TestLength

    @test_length.setter
    def test_length(self, value: int):
        self.bert.TestLength = value

    @property
    def logging_mode(self) -> bool:
        return self.bert.LoggingMode

    @logging_mode.setter
    def logging_mode(self, value: bool):
        self.bert.LoggingMode = value

    @property
    def software_mode(self) -> bool:
        return self.bert.softwareMode

    @software_mode.setter
    def software_mode(self, value: bool):
        self.bert.softWareMode = value

    @property
    def DTR(self) -> bool:
        return self.bert.DTR

    @DTR.setter
    def DTR(self, value: bool):
        self.bert.DTR = value

    @property
    def RTS(self) -> bool:
        return self.bert.RTS

    @RTS.setter
    def RTS(self, value: bool):
        self.bert.RTS = value

    @property
    def bit_count(self) -> float:
        return self.bert.BitCount

    @property
    def bit_error_rate(self) -> float:
        return self.bert.BitErrorRate

    @property
    def status(self) -> int:
        return self.bert.Status

    @property
    def consts(self) -> Constants:
        return Constants(self.bert)

    def inject_errors(self, number_of_errors):
        self.bert.InjectError(number_of_errors)

    def reset(self):
        self.bert.Reset()

    def restart(self):
        self.stop()
        self.start()

    def start(self):
        self.bert.Start()

    def stop(self):
        self.bert.Stop()
